@@ -1,24 +1,24 @@
 import os
-from distutils.core import setup
+from setuptools import Extension, setup
 from Cython.Build import cythonize
-from distutils.extension import Extension
 
 from build_utils import set_cc_variables
 from build_utils import get_openmp_flag
 
-
 original_environ = os.environ.copy()
 try:
     set_cc_variables("CC_OUTER_LOOP")
     openmp_flag = get_openmp_flag()
 
-    use_blis = os.getenv("INSTALL_BLIS", False)
-    libraries = ["blis"] if use_blis else []
+    use_custom_blas = os.getenv("INSTALL_BLAS", False)
+    libraries = [use_custom_blas] if use_custom_blas else []
+    custom_suffix = "_custom" if use_custom_blas else ""
+    filename = f"nested_prange_blas{custom_suffix}.pyx"
 
     ext_modules = [
         Extension(
             "nested_prange_blas",
-            ["nested_prange_blas.pyx"],
+            [filename],
             extra_compile_args=openmp_flag,
             extra_link_args=openmp_flag,
             libraries=libraries,
@@ -29,7 +29,6 @@ try:
         name="_openmp_test_helper_nested_prange_blas",
         ext_modules=cythonize(
             ext_modules,
-            compile_time_env={"USE_BLIS": use_blis},
             compiler_directives={
                 "language_level": 3,
                 "boundscheck": False,
