@@ -10,6 +10,7 @@ from threadpoolctl import ThreadpoolController
 from threadpoolctl import _ALL_PREFIXES, _ALL_USER_APIS
 
 from .utils import cython_extensions_compiled
+from .utils import check_nested_prange_blas
 from .utils import libopenblas_paths
 from .utils import scipy
 from .utils import threadpool_info_from_subprocess
@@ -22,6 +23,21 @@ def is_old_openblas(lib_controller):
     return lib_controller.internal_api == "openblas" and lib_controller.version is None
 
 
+def skip_if_openblas_openmp():
+    """Helper to skip tests with side effects when OpenBLAS has the OpenMP
+    threading layer.
+    """
+    if any(
+        lib_controller.internal_api == "openblas"
+        and lib_controller.threading_layer == "openmp"
+        for lib_controller in ThreadpoolController().lib_controllers
+    ):
+        pytest.skip(
+            "Setting a limit on OpenBLAS when using the OpenMP threading layer also "
+            "impact the OpenMP library. They can't be controlled independently."
+        )
+
+
 def effective_num_threads(nthreads, max_threads):
     if nthreads is None or nthreads > max_threads:
         return max_threads
@@ -196,6 +212,10 @@ def test_threadpool_limits_manual_restore():
 def test_threadpool_controller_limit():
     # Check that using the limit method of ThreadpoolController only impact its
     # library controllers.
+
+    # This is not True for OpenBLAS with the OpenMP threading layer.
+    skip_if_openblas_openmp()
+
     blas_controller = ThreadpoolController().select(user_api="blas")
     original_openmp_info = ThreadpoolController().select(user_api="openmp").info()
 
@@ -208,14 +228,8 @@ def test_threadpool_controller_limit():
             for lib_controller in blas_controller.lib_controllers
         )
         # original_blas_controller contains only blas libraries so no opemp library
-        # should be impacted. This is not True for OpenBLAS with the OpenMP threading
-        # layer.
-        if not any(
-            lib_controller.internal_api == "openblas"
-            and lib_controller.threading_layer == "openmp"
-            for lib_controller in blas_controller.lib_controllers
-        ):
-            assert openmp_info == original_openmp_info
+        # should be impacted.
+        assert openmp_info == original_openmp_info
 
 
 def test_get_params_for_sequential_blas_under_openmp():
@@ -393,19 +407,22 @@ def test_multiple_shipped_openblas():
     test_shipped_openblas()
 
 
-@pytest.mark.skipif(scipy is None, reason="requires scipy")
 @pytest.mark.skipif(
     not cython_extensions_compiled, reason="Requires cython extensions to be compiled"
 )
+@pytest.mark.skipif(
+    check_nested_prange_blas is None,
+    reason="Requires nested_prange_blas to be compiled",
+)
 @pytest.mark.parametrize("nthreads_outer", [None, 1, 2, 4])
 def test_nested_prange_blas(nthreads_outer):
-    # Check that the BLAS linked to scipy effectively uses the number of
-    # threads requested by the context manager when nested in an outer OpenMP
-    # loop.
+    # Check that the BLAS uses the number of threads requested by the context manager
+    # when nested in an outer OpenMP loop.
+    # Remark: this test also works with sequential BLAS only because we limit the
+    # number of threads for the BLAS to 1.
     import numpy as np
-    import tests._openmp_test_helper.nested_prange_blas as prange_blas
 
-    check_nested_prange_blas = prange_blas.check_nested_prange_blas
+    skip_if_openblas_openmp()
 
     original_info = ThreadpoolController().info()
 
@@ -445,7 +462,7 @@ def test_nested_prange_blas(nthreads_outer):
 
 # the method `get_original_num_threads` raises a UserWarning due to different
 # num_threads from libraries with the same `user_api`. It will be raised only
-# in the CI job with 2 openblas (py37_pip_openblas_gcc_clang). It is expected
+# in the CI job with 2 openblas (py38_pip_openblas_gcc_clang). It is expected
 # so we can safely filter it.
 @pytest.mark.filterwarnings("ignore::UserWarning")
 @pytest.mark.parametrize("limit", [1, None])
@@ -530,9 +547,20 @@ def test_libomp_libiomp_warning(recwarn):
     assert "multiple_openmp.md" in str(wm.message)
 
 
-def test_command_line_empty():
+def test_command_line_empty_or_system_openmp():
+    # When the command line is called without arguments, no library should be
+    # detected. The only exception is a system OpenMP library that can be
+    # linked to the Python interpreter, for instance via the libb2.so BLAKE2
+    # library that can itself be linked to an OpenMP runtime on Gentoo.
     output = subprocess.check_output((sys.executable + " -m threadpoolctl").split())
-    assert json.loads(output.decode("utf-8")) == []
+    results = json.loads(output.decode("utf-8"))
+    conda_prefix = os.getenv("CONDA_PREFIX")
+    managed_by_conda = conda_prefix and sys.executable.startswith(conda_prefix)
+    if not managed_by_conda:  # pragma: no cover
+        # When using a Python interpreter that does not come from a conda
+        # environment, we should ignore any system OpenMP library.
+        results = [r for r in results if r["user_api"] != "openmp"]
+    assert results == []
 
 
 def test_command_line_command_flag():
@@ -585,23 +613,24 @@ def test_architecture():
     expected_openblas_architectures = (
         # XXX: add more as needed by CI or developer laptops
         "armv8",
-        "Haswell",
-        "Prescott",  # see: https://github.com/xianyi/OpenBLAS/pull/3485
-        "SkylakeX",
-        "Sandybridge",
-        "VORTEX",
-        "Zen",
+        "haswell",
+        "prescott",  # see: https://github.com/xianyi/OpenBLAS/pull/3485
+        "skylakex",
+        "sandybridge",
+        "vortex",
+        "zen",
     )
     expected_blis_architectures = (
         # XXX: add more as needed by CI or developer laptops
         "skx",
         "haswell",
+        "zen3",
     )
     for lib_info in threadpool_info():
         if lib_info["internal_api"] == "openblas":
-            assert lib_info["architecture"] in expected_openblas_architectures
+            assert lib_info["architecture"].lower() in expected_openblas_architectures
         elif lib_info["internal_api"] == "blis":
-            assert lib_info["architecture"] in expected_blis_architectures
+            assert lib_info["architecture"].lower() in expected_blis_architectures
         else:
             # Not supported for other libraries
             assert "architecture" not in lib_info
@@ -626,6 +655,81 @@ def test_openblas_threading_layer():
     assert threading_layer in expected_openblas_threading_layers
 
 
+# skip test if not run in a azure pipelines job since it relies on a specific flexiblas
+# installation.
+@pytest.mark.skipif(
+    "GITHUB_ACTIONS" not in os.environ, reason="not running in azure pipelines"
+)
+def test_flexiblas():
+    # Check that threadpool_info correctly recovers the FlexiBLAS backends.
+    flexiblas_controller = ThreadpoolController().select(internal_api="flexiblas")
+
+    if not flexiblas_controller:
+        pytest.skip("requires FlexiBLAS.")
+    flexiblas_controller = flexiblas_controller.lib_controllers[0]
+
+    expected_backends = {"NETLIB", "OPENBLAS_CONDA"}
+    expected_backends_loaded = {"OPENBLAS_CONDA"}
+    expected_current_backend = "OPENBLAS_CONDA"  # set as default at build time
+
+    flexiblas_backends = flexiblas_controller.available_backends
+    flexiblas_backends_loaded = flexiblas_controller.loaded_backends
+    current_backend = flexiblas_controller.current_backend
+
+    assert set(flexiblas_backends) == expected_backends
+    assert set(flexiblas_backends_loaded) == expected_backends_loaded
+    assert current_backend == expected_current_backend
+
+
+def test_flexiblas_switch_error():
+    # Check that an error is raised when trying to switch to an invalid backend.
+    flexiblas_controller = ThreadpoolController().select(internal_api="flexiblas")
+
+    if not flexiblas_controller:
+        pytest.skip("requires FlexiBLAS.")
+    flexiblas_controller = flexiblas_controller.lib_controllers[0]
+
+    with pytest.raises(RuntimeError, match="Failed to load backend"):
+        flexiblas_controller.switch_backend("INVALID_BACKEND")
+
+
+# skip test if not run in a azure pipelines job since it relies on a specific flexiblas
+# installation.
+@pytest.mark.skipif(
+    "GITHUB_ACTIONS" not in os.environ, reason="not running in azure pipelines"
+)
+def test_flexiblas_switch():
+    # Check that the backend can be switched.
+    controller = ThreadpoolController()
+    fb_controller = controller.select(internal_api="flexiblas")
+
+    if not fb_controller:
+        pytest.skip("requires FlexiBLAS.")
+    fb_controller = fb_controller.lib_controllers[0]
+
+    # at first mkl is not loaded in the CI jobs where this test runs
+    assert len(controller.select(internal_api="mkl").lib_controllers) == 0
+
+    # at first, only "OPENBLAS_CONDA" is loaded
+    assert fb_controller.current_backend == "OPENBLAS_CONDA"
+    assert fb_controller.loaded_backends == ["OPENBLAS_CONDA"]
+
+    fb_controller.switch_backend("NETLIB")
+    assert fb_controller.current_backend == "NETLIB"
+    assert fb_controller.loaded_backends == ["OPENBLAS_CONDA", "NETLIB"]
+
+    if sys.platform == "linux":
+        mkl_path = f"{os.getenv('CONDA_PREFIX')}/lib/libmkl_rt.so"
+        fb_controller.switch_backend(mkl_path)
+        assert fb_controller.current_backend == mkl_path
+        assert fb_controller.loaded_backends == ["OPENBLAS_CONDA", "NETLIB", mkl_path]
+        # switching the backend triggered a new search for loaded shared libs
+        assert len(controller.select(internal_api="mkl").lib_controllers) == 1
+
+    # switch back to default to avoid side effects
+    fb_controller.switch_backend("OPENBLAS_CONDA")
+
+
 def test_threadpool_controller_as_decorator():
     # Check that using the decorator can be nested and is restricted to the scope of
     # the decorated function.
@@ -657,3 +761,33 @@ def test_threadpool_controller_as_decorator():
     outer_func()
 
     assert ThreadpoolController().info() == original_info
+
+
+def test_custom_controller():
+    # Check that a custom controller can be used to change the number of threads
+    # used by a library.
+    try:
+        import tests._pyMylib  # noqa
+    except:
+        pytest.skip("requires my_thread_lib to be compiled")
+
+    controller = ThreadpoolController()
+    original_info = controller.info()
+
+    mylib_controller = controller.select(user_api="my_threaded_lib")
+
+    # my_threaded_lib has been found and there's 1 matching shared library
+    assert len(mylib_controller.lib_controllers) == 1
+    mylib_controller = mylib_controller.lib_controllers[0]
+
+    # we linked against my_threaded_lib v2.0 and by default it uses 42 thread
+    assert mylib_controller.version == "2.0"
+    assert mylib_controller.num_threads == 42
+
+    # my_threaded_lib exposes an additional info "some_attr":
+    assert mylib_controller.info()["some_attr"] == "some_value"
+
+    with controller.limit(limits=1, user_api="my_threaded_lib"):
+        assert mylib_controller.num_threads == 1
+
+    assert ThreadpoolController().info() == original_info
